"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.readJsonFiles = exports.walk = exports.getDeployedContracts = exports.getDeployedChains = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const chains_1 = require("../models/chains");
async function getDeployedChains(_path) {
    const list = fs_1.default.readdirSync(_path);
    let chains = [];
    for (const folder of list) {
        const folderPath = path_1.default.join(_path, folder);
        const stat = fs_1.default.statSync(folderPath);
        if (stat && stat.isDirectory()) {
            const name = folder;
            const chainIdPath = path_1.default.join(folderPath, ".chainId");
            const chainId = fs_1.default.readFileSync(chainIdPath, "utf8");
            const chain = new chains_1.ChainInfo(+chainId, folderPath, name);
            chains.push(chain);
        }
    }
    return chains;
}
exports.getDeployedChains = getDeployedChains;
async function getDeployedContracts(_path) {
    const list = fs_1.default.readdirSync(_path);
    let contracts = [];
    for (const file of list) {
        const filePath = path_1.default.join(_path, file);
        const stat = fs_1.default.statSync(filePath);
        if (stat && stat.isFile()) {
            if (file.endsWith(".json")) {
                const name = file.replace(".json", "");
                const contractJsonFile = fs_1.default.readFileSync(filePath, "utf8");
                const contractJson = JSON.parse(contractJsonFile);
                const address = contractJson.address;
                const abi = contractJson.abi;
                const contract = new chains_1.ContractInfo(address, filePath, name, abi);
                contracts.push(contract);
            }
        }
    }
    return contracts;
}
exports.getDeployedContracts = getDeployedContracts;
async function walk(dir) {
    let results = [];
    const list = fs_1.default.readdirSync(dir);
    for (const file of list) {
        const filePath = path_1.default.join(dir, file);
        const stat = fs_1.default.statSync(filePath);
        if (stat && stat.isDirectory()) {
            const res = await walk(filePath);
            results = results.concat(res);
        }
        else {
            if (file.endsWith(".dbg.json")) {
            }
            else {
                results.push(filePath);
            }
        }
    }
    return results;
}
exports.walk = walk;
function readJsonFiles(files) {
    const result = [];
    for (const file of files) {
        const data = fs_1.default.readFileSync(file, "utf8");
        const { abi, contractName, sourceName } = JSON.parse(data);
        const params = {
            path: file,
            sourceName,
            contractName,
            abi,
        };
        result.push(params);
    }
    return result;
}
exports.readJsonFiles = readJsonFiles;
//# sourceMappingURL=helpers.js.map